import numpy as np
import os
from qtpy import QtGui, QtCore, QtWidgets
import sharppy.sharptab as tab
import sharppy.databases.inset_data as inset_data
from sharppy.sharptab.constants import *
import platform 

## routine written by Kelton Halbert and Greg Blumberg
## keltonhalbert@ou.edu and wblumberg@ou.edu

__all__ = ['backgroundSHIP', 'plotSHIP']

class backgroundSHIP(QtWidgets.QFrame):
    '''
    Draw the background frame and lines for the Theta-E plot frame
    '''
    def __init__(self):
        super(backgroundSHIP, self).__init__()
        self.initUI()


    def initUI(self):
        ## window configuration settings,
        ## sich as padding, width, height, and
        ## min/max plot axes
        self.setStyleSheet("QFrame {"
            "  background-color: rgb(0, 0, 0);"
            "  border-width: 1px;"
            "  border-style: solid;"
            "  border-color: #3399CC;}")
        self.textpad = 5
        self.font_ratio = 0.0512
        fsize1 = round(self.size().height() * self.font_ratio) + 2
        fsize2 = round(self.size().height() * self.font_ratio)
        self.plot_font = QtGui.QFont('Helvetica', fsize1 )
        self.box_font = QtGui.QFont('Helvetica', fsize2)
        self.plot_metrics = QtGui.QFontMetrics( self.plot_font )
        self.box_metrics = QtGui.QFontMetrics(self.box_font)
        if platform.system() == "Windows":
            fsize1 -= self.plot_metrics.descent()
            fsize2 -= self.box_metrics.descent()

            self.plot_font = QtGui.QFont('Helvetica', fsize1 )
            self.box_font = QtGui.QFont('Helvetica', fsize2)
            self.plot_metrics = QtGui.QFontMetrics( self.plot_font )
            self.box_metrics = QtGui.QFontMetrics(self.box_font)
        self.plot_height = self.plot_metrics.xHeight()# + self.textpad
        self.box_height = self.box_metrics.xHeight() + self.textpad
        self.tpad = self.plot_height + 15; 
        self.bpad = self.plot_height + 2
        self.lpad = 0.; self.rpad = 0.
        self.wid = self.size().width() - self.rpad
        self.hgt = self.size().height() - self.bpad
        self.tlx = self.rpad; self.tly = self.tpad;
        self.brx = self.wid; 
        self.bry = self.hgt - self.bpad#+ round(self.font_ratio * self.hgt)
        self.shipmax = 5.; self.shipmin = 0.
        self.plotBitMap = QtGui.QPixmap(self.width()-2, self.height()-2)
        self.plotBitMap.fill(self.bg_color)
        self.plotBackground()

    def resizeEvent(self, e):
        '''
        Handles the event the window is resized
        '''
        self.initUI()
    
    def plotBackground(self):
        '''
        Handles painting the frame.
        '''
        ## initialize a painter object and draw the frame
        qp = QtGui.QPainter()
        qp.begin(self.plotBitMap)
        qp.setRenderHint(qp.Antialiasing)
        qp.setRenderHint(qp.TextAntialiasing)
        self.draw_frame(qp)
        qp.end()

    def draw_frame(self, qp):
        '''
        Draw the background frame.
        qp: QtGui.QPainter object
        '''
        ## set a new pen to draw with
        pen = QtGui.QPen(self.fg_color, 2, QtCore.Qt.SolidLine)
        qp.setPen(pen)
        qp.setFont(self.plot_font)
        rect1 = QtCore.QRectF(1.5,6, self.brx, self.plot_height)
        qp.drawText(rect1, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter,
            'Significant Hail Param (SHIP)')

        pen = QtGui.QPen(QtCore.Qt.blue, 1, QtCore.Qt.DashLine)
        qp.setPen(pen)
        spacing = self.bry / 6.

        ytick_fontsize = round(self.font_ratio * self.hgt)
        y_ticks_font = QtGui.QFont('Helvetica', ytick_fontsize)
        qp.setFont(y_ticks_font)
        ship_inset_data = inset_data.shipData()
        texts = ship_inset_data['ship_ytexts']
        for i in range(len(texts)):
            pen = QtGui.QPen(self.line_color, 1, QtCore.Qt.DashLine)
            qp.setPen(pen)
            try:
                qp.drawLine(self.tlx, self.ship_to_pix(int(texts[i])), self.brx, self.ship_to_pix(int(texts[i])))
            except:
                continue
            color = QtGui.QColor(self.bg_color)
            pen = QtGui.QPen(color, 1, QtCore.Qt.SolidLine)
            qp.setPen(pen)
            ypos = spacing*(i+1) - (spacing/4.)
            ypos = self.ship_to_pix(int(texts[i])) - ytick_fontsize/2
            rect = QtCore.QRect(self.tlx, ypos, 20, ytick_fontsize)
            pen = QtGui.QPen(self.fg_color, 1, QtCore.Qt.SolidLine)
            qp.setPen(pen)
            qp.drawText(rect, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter, texts[i])

        ef = ship_inset_data['ship_dist']
        width = self.brx / 3.7
        spacing = self.brx / 3
        center = np.arange(spacing, self.brx, spacing)
        texts = ship_inset_data['ship_xtexts']
        ef = self.ship_to_pix(ef)
        qp.setFont(QtGui.QFont('Helvetica', round(self.font_ratio * self.hgt)))
        for i in range(ef.shape[0]):
            # Set green pen to draw box and whisker plots 
            pen = QtGui.QPen(self.box_color, 2, QtCore.Qt.SolidLine)
            qp.setPen(pen)
            # Draw lower whisker
            qp.drawLine(center[i], ef[i,0], center[i], ef[i,1])
            # Draw box
            qp.drawLine(center[i] - width/2., ef[i,3], center[i] + width/2., ef[i,3])
            qp.drawLine(center[i] - width/2., ef[i,1], center[i] + width/2., ef[i,1])
            qp.drawLine(center[i] - width/2., ef[i,1], center[i] - width/2., ef[i,3])
            qp.drawLine(center[i] + width/2., ef[i,1], center[i] + width/2., ef[i,3])
            # Draw median
            #qp.drawLine(center[i] - width/2., ef[i,2], center[i] + width/2., ef[i,2])
            # Draw upper whisker
            qp.drawLine(center[i], ef[i,3], center[i], ef[i,4])
            # Set black transparent pen to draw a rectangle
            color = QtGui.QColor(self.bg_color)
            color.setAlpha(0)
            pen = QtGui.QPen(color, 1, QtCore.Qt.SolidLine)
            rect = QtCore.QRectF(center[i] - width/2., self.bry + self.bpad/2, width, self.bpad)
            # Change to a white pen to draw the text below the box and whisker plot
            pen = QtGui.QPen(self.fg_color, 1, QtCore.Qt.SolidLine)
            qp.setPen(pen)
            qp.drawText(rect, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter, texts[i])

    def ship_to_pix(self, ship):
        scl1 = self.shipmax - self.shipmin
        scl2 = self.shipmin + ship
        return self.bry - (scl2 / scl1) * (self.bry - self.tpad)


class plotSHIP(backgroundSHIP):
    '''
    Plot the data on the frame. Inherits the background class that
    plots the frame.
    '''
    def __init__(self):
        self.bg_color = QtGui.QColor('#000000')
        self.fg_color = QtGui.QColor('#ffffff')
        self.box_color = QtGui.QColor('#00ff00')
        self.line_color = QtGui.QColor('#0080ff')

        self.alert_colors = [
            QtGui.QColor('#775000'),
            QtGui.QColor('#996600'),
            QtGui.QColor('#ffffff'),
            QtGui.QColor('#ffff00'),
            QtGui.QColor('#ff0000'),
            QtGui.QColor('#e700df'),
        ]

        super(plotSHIP, self).__init__()
        self.prof = None

    def setProf(self, prof):
        self.prof = prof
        self.ship = prof.ship

        self.clearData()
        self.plotBackground()
        self.plotData()
        self.update()

    def setPreferences(self, update_gui=True, **prefs):
        self.bg_color = QtGui.QColor(prefs['bg_color'])
        self.fg_color = QtGui.QColor(prefs['fg_color'])
        self.box_color = QtGui.QColor(prefs['stp_box_color'])
        self.line_color = QtGui.QColor(prefs['stp_line_color'])

        self.alert_colors = [
            QtGui.QColor(prefs['alert_l1_color']),
            QtGui.QColor(prefs['alert_l2_color']),
            QtGui.QColor(prefs['alert_l3_color']),
            QtGui.QColor(prefs['alert_l4_color']),
            QtGui.QColor(prefs['alert_l5_color']),
            QtGui.QColor(prefs['alert_l6_color']),
        ]

        if update_gui:
            self.clearData()
            self.plotBackground()
            self.plotData()
            self.update()

    def resizeEvent(self, e):
        '''
        Handles when the window is resized
        '''
        super(plotSHIP, self).resizeEvent(e)
        self.plotData()
    
    def paintEvent(self, e):
        super(plotSHIP, self).paintEvent(e)
        qp = QtGui.QPainter()
        qp.begin(self)
        qp.drawPixmap(1, 1, self.plotBitMap)
        qp.end()

    def clearData(self):
        '''
        Handles the clearing of the pixmap
        in the frame.
        '''
        self.plotBitMap = QtGui.QPixmap(self.width(), self.height())
        self.plotBitMap.fill(self.bg_color)
    
    def plotData(self):
        '''
        Handles painting on the frame
        '''
        if self.prof is None:
            return

        ## this function handles painting the plot
        ## create a new painter obkect
        qp = QtGui.QPainter()
        self.draw_ship(qp)

    def draw_ship(self, qp):
        if not tab.utils.QC(self.ship):
            return

        qp.begin(self.plotBitMap)
        qp.setRenderHint(qp.Antialiasing)
        qp.setRenderHint(qp.TextAntialiasing)

        if self.ship < self.shipmin:
            self.ship = self.shipmin
        elif self.ship > self.shipmax:
            self.ship = self.shipmax

        color = self.ship_color(self.ship) 
        ef = self.ship_to_pix(self.ship)
        pen = QtGui.QPen(color, 1.5, QtCore.Qt.SolidLine)
        qp.setPen(pen)
        qp.drawLine(0, ef, self.wid, ef)
        qp.end()

    def ship_color(self, ship):
        color_list = self.alert_colors
        if float(ship) >= 5:
            color = color_list[5]
        elif float(ship) >= 2:
            color = color_list[4]
        elif float(ship) >= 1:
            color = color_list[3]
        elif float(ship) >= .5:
            color = color_list[2]
        else:
            color = color_list[0]

        return color

if __name__ == '__main__':
    app_frame = QtGui.QApplication([])    
    tester = plotSHIP()
    tester.show()    
    app_frame.exec_()



import numpy as np
from qtpy import QtGui, QtCore, QtWidgets
import sharppy.sharptab as tab
import sharppy.sharptab.utils as utils
from sharppy.sharptab.constants import *

## routine written by Kelton Halbert
## keltonhalbert@ou.edu

__all__ = ['backgroundWinds', 'plotWinds']

class backgroundWinds(QtWidgets.QFrame):
    '''
    Handles the plotting of the frame boarders and ticks.
    Draws the frame onto a QPixmap.
    '''
    def __init__(self):
        super(backgroundWinds, self).__init__()
        self.initUI()


    def initUI(self):
        '''
        Initializes the frame attributes and QPixmap.
        '''
        ## initialize plot window variables,
        ## such as width, height, padding, and
        ## min/max axes variables
        self.lpad = 0; self.rpad = 0
        self.tpad = 0; self.bpad = 20
        self.wid = self.size().width() - self.rpad
        self.hgt = self.size().height() - self.bpad
        self.tlx = self.rpad; self.tly = self.tpad
        self.brx = self.wid; self.bry = self.hgt
        ## minimum/maximum height (km) and wind values.
        ## used in converting to pixel units
        self.hmax = 16.; self.hmin = 0.
        self.smax = 80.; self.smin = 0.
        self.font_ratio = 0.0512
        self.label_font = QtGui.QFont('Helvetica', round(self.size().height() * self.font_ratio))
        ## initialize the QPixmap
        self.plotBitMap = QtGui.QPixmap(self.width(), self.height())
        self.plotBitMap.fill(self.bg_color)
        self.plotBackground()

    def resizeEvent(self, e):
        '''
        Handles the event the window is resized.
        '''
        self.initUI()
    
    def plotBackground(self):
        '''
        This paints the frame background onto the QPixmap.
        '''
        ## initialize a QPainter object for drawing
        qp = QtGui.QPainter()
        qp.begin(self.plotBitMap)
        qp.setRenderHint(qp.Antialiasing)
        qp.setRenderHint(qp.TextAntialiasing)
        ## draw the background frame
        self.draw_frame(qp)
        ## draw the ticks for the plot.
        ## height is in km.
        for h in [2,4,6,8,10,12,14]:
            self.draw_height(h, qp)
        for s in range(0,100,10):
            self.draw_speed(s, qp)
        qp.end()

    def draw_frame(self, qp):
        '''
        Draws the background frame.
        
        Parameters
        ----------
        qp: QtGui.QPainter object
        
        '''
        ## initialize a new pen with white color, thickness of 2, solid line.
        pen = QtGui.QPen(self.fg_color, 2, QtCore.Qt.SolidLine)
        qp.setPen(pen)
        qp.setFont(self.label_font)
        qp.drawText(15, 5, 45, 35,
            QtCore.Qt.AlignVCenter | QtCore.Qt.AlignHCenter,
            'SR Wind\nv.\nHeight')
        ## draw the frame borders
        qp.drawLine(self.tlx, self.tly, self.brx, self.tly)
        qp.drawLine(self.brx, self.tly, self.brx, self.bry)
        qp.drawLine(self.brx, self.bry, self.tlx, self.bry)
        qp.drawLine(self.tlx, self.bry, self.tlx, self.tly)
        pen = QtGui.QPen(self.fg_color, 1, QtCore.Qt.DashLine)
        qp.setPen(pen)
        zero = self.speed_to_pix(15.)
        qp.drawLine( zero, self.bry, zero, self.tly)
        lower = self.hgt_to_pix(8.)
        upper = self.hgt_to_pix(16.)
        classic1 = self.speed_to_pix(40.)
        classic2 = self.speed_to_pix(70.)
        pen = QtGui.QPen(self.clsc_color, 1, QtCore.Qt.DashLine)
        qp.setPen(pen)
        qp.drawLine( classic1, lower, classic1, upper )
        qp.drawLine( classic2, lower, classic2, upper )
        qp.drawText(classic1-5, 2, 50, 50,
            QtCore.Qt.AlignVCenter | QtCore.Qt.AlignHCenter,
            'Classic\nSupercell')
        ## draw the plot description text

    def draw_height(self, h, qp):
        '''
        Draw background height ticks in km.
        
        Parameters
        ----------
        h: height in km
        qp: QtGui.QPainter object
        
        '''
        ## initialize a pen with color white, thickness 1, solid line
        pen = QtGui.QPen(self.fg_color, 1, QtCore.Qt.SolidLine)
        qp.setPen(pen)
        qp.setFont(self.label_font)
        ## convert the height value to pixel coordinates
        y1 = self.hgt_to_pix(h)
        ## length of tick to be drawn
        offset = 5
        ## draw the tick and then label the tick with the value
        qp.drawLine(self.lpad, y1, self.lpad+offset, y1)
        qp.drawLine(self.brx+self.rpad-offset, y1,
            self.brx+self.rpad, y1)
        qp.drawText(0, y1-20, 20, 40,
            QtCore.Qt.AlignVCenter | QtCore.Qt.AlignRight,
            utils.INT2STR(h))

    def draw_speed(self, s, qp):
        '''
        Draw background speed ticks.
        
        Parameters
        ----------
        s: windpeed
        qp: QtGui.QPainter object
        
        '''
        ## initialize a pen with white color, thickness 1, solid line
        pen = QtGui.QPen(self.fg_color, 1, QtCore.Qt.SolidLine)
        qp.setPen(pen)
        qp.setFont(self.label_font)
        ## convert wind speed values to pixel values
        x1 = self.speed_to_pix(s)
        ## length of tick to be drawn
        offset = 5
        qp.drawLine(x1, 0, x1, 0+offset)
        qp.drawLine(x1, self.bry+self.tpad-offset,
            x1, self.bry+self.rpad)

    def hgt_to_pix(self, h):
        '''
        Function to convert a height value (km) to a Y pixel.
        
        Parameters
        ----------
        h: height in km
        
        '''
        scl1 = self.hmax - self.hmin
        scl2 = self.hmin + h
        return (self.bry - 2) - (scl2 / scl1) * (self.bry - self.tpad)

    def speed_to_pix(self, s):
        '''
        Function to convert a wind speed value to a X pixel.
        
        Parameters
        ----------
        s: speed in kts
        '''
        scl1 = self.smax - self.smin
        scl2 = self.smax - s
        return self.bry - (scl2 / scl1) * (self.bry - self.rpad)

class plotWinds(backgroundWinds):
    '''
    Draws the storm relative winds vs. height.
    Inherits from the backgroundWinds class that 
    contains the QPixmap with the background frame
    drawn on it.
    '''
    def __init__(self):
        '''
        Initializes the data from the Profile object
        used to draw the profile.
        
        Parameters
        ----------
        prof: a Profile object
        
        '''
        self.bg_color = QtGui.QColor('#000000')
        self.fg_color = QtGui.QColor('#ffffff')
        self.clsc_color = QtGui.QColor('#b1019a')
        self.trace_color = QtGui.QColor('#ff0000')
        self.m0_2_color = QtGui.QColor('#8b0000')
        self.m4_6_color = QtGui.QColor('#6495ed')
        self.m9_11_color = QtGui.QColor('#9400d3')

        self.use_left = False

        super(plotWinds, self).__init__()
        ## make the data accessable to the functions
        self.prof = None

    def setProf(self, prof):
        self.prof = prof

        self.u = prof.u; self.v = prof.v
        ## calculate the storm relative wind from the bunkers motion function
        self.srwind = prof.srwind

        if self.use_left:
            self.srw_0_2km = utils.comp2vec(self.prof.left_srw_0_2km[0], self.prof.left_srw_0_2km[1])[1]
            self.srw_4_6km = utils.comp2vec(self.prof.left_srw_4_6km[0], self.prof.left_srw_4_6km[1])[1]
            self.srw_9_11km = utils.comp2vec(self.prof.left_srw_9_11km[0], self.prof.left_srw_9_11km[1])[1]
            ## get only the left mover u and v components
            self.sru = self.u - self.srwind[2]
            self.srv = self.v - self.srwind[3]
        else:
            self.srw_0_2km = utils.comp2vec(self.prof.right_srw_0_2km[0], self.prof.right_srw_0_2km[1])[1]
            self.srw_4_6km = utils.comp2vec(self.prof.right_srw_4_6km[0], self.prof.right_srw_4_6km[1])[1]
            self.srw_9_11km = utils.comp2vec(self.prof.right_srw_9_11km[0], self.prof.right_srw_9_11km[1])[1]
            ## get only the right mover u and v components
            self.sru = self.u - self.srwind[0]
            self.srv = self.v - self.srwind[1]

        self.clearData()
        self.plotBackground()
        self.plotData()
        self.update()

    def setPreferences(self, update_gui=True, **prefs):
        self.bg_color = QtGui.QColor(prefs['bg_color'])
        self.fg_color = QtGui.QColor(prefs['fg_color'])
        self.clsc_color = QtGui.QColor(prefs['srw_clsc_color'])
        self.trace_color = QtGui.QColor(prefs['srw_trace_color'])
        self.m0_2_color = QtGui.QColor(prefs['srw_0_2_color'])
        self.m4_6_color = QtGui.QColor(prefs['srw_4_6_color'])
        self.m9_11_color = QtGui.QColor(prefs['srw_9_11_color'])

        if update_gui:
            if self.use_left:
                self.srw_0_2km = utils.comp2vec(self.prof.left_srw_0_2km[0], self.prof.left_srw_0_2km[1])[1]
                self.srw_4_6km = utils.comp2vec(self.prof.left_srw_4_6km[0], self.prof.left_srw_4_6km[1])[1]
                self.srw_9_11km = utils.comp2vec(self.prof.left_srw_9_11km[0], self.prof.left_srw_9_11km[1])[1]
                ## get only the left mover u and v components
                self.sru = self.u - self.srwind[2]
                self.srv = self.v - self.srwind[3]
            else:
                self.srw_0_2km = utils.comp2vec(self.prof.right_srw_0_2km[0], self.prof.right_srw_0_2km[1])[1]
                self.srw_4_6km = utils.comp2vec(self.prof.right_srw_4_6km[0], self.prof.right_srw_4_6km[1])[1]
                self.srw_9_11km = utils.comp2vec(self.prof.right_srw_9_11km[0], self.prof.right_srw_9_11km[1])[1]
                ## get only the right mover u and v components
                self.sru = self.u - self.srwind[0]
                self.srv = self.v - self.srwind[1]

            self.clearData()
            self.plotBackground()
            self.plotData()
            self.update()

    def setDeviant(self, deviant):
        self.use_left = deviant == 'left'

        if self.use_left:
            self.srw_0_2km = utils.comp2vec(self.prof.left_srw_0_2km[0], self.prof.left_srw_0_2km[1])[1]
            self.srw_4_6km = utils.comp2vec(self.prof.left_srw_4_6km[0], self.prof.left_srw_4_6km[1])[1]
            self.srw_9_11km = utils.comp2vec(self.prof.left_srw_9_11km[0], self.prof.left_srw_9_11km[1])[1]
            ## get only the left mover u and v components
            self.sru = self.u - self.srwind[2]
            self.srv = self.v - self.srwind[3]
        else:
            self.srw_0_2km = utils.comp2vec(self.prof.right_srw_0_2km[0], self.prof.right_srw_0_2km[1])[1]
            self.srw_4_6km = utils.comp2vec(self.prof.right_srw_4_6km[0], self.prof.right_srw_4_6km[1])[1]
            self.srw_9_11km = utils.comp2vec(self.prof.right_srw_9_11km[0], self.prof.right_srw_9_11km[1])[1]
            ## get only the right mover u and v components
            self.sru = self.u - self.srwind[0]
            self.srv = self.v - self.srwind[1]

        self.clearData()
        self.plotBackground()
        self.plotData()
        self.update()

    def resizeEvent(self, e):
        '''
        Handles when the window is resized.
        
        Parameters
        ----------
        e: an Event object
        
        '''
        super(plotWinds, self).resizeEvent(e)
        self.plotData()
    
    def paintEvent(self, e):
        '''
        Handles the painting of the QPixmap onto
        the QWidget.
        
        Parameters
        ----------
        e: an Event object
        
        '''
        super(plotWinds, self).paintEvent(e)
        qp = QtGui.QPainter()
        qp.begin(self)
        qp.drawPixmap(0, 0, self.plotBitMap)
        qp.end()

    def clearData(self):
        '''
        Handles the clearing of the pixmap
        in the frame.
        '''
        self.plotBitMap = QtGui.QPixmap(self.width(), self.height())
        self.plotBitMap.fill(self.bg_color)
    
    def plotData(self):
        '''
        Handles painting the plot data onto the
        QPixmap.
        '''
        if self.prof is None:
            return

        ## initialize a QPainter objext
        qp = QtGui.QPainter()
        qp.begin(self.plotBitMap)
        qp.setRenderHint(qp.Antialiasing)
        qp.setRenderHint(qp.TextAntialiasing)
        ## draw the wind profile
        self.draw_profile(qp)
        qp.end()

    def draw_profile(self, qp):
        '''
        Draws the storm relative wind profile.
        
        Parameters
        ----------
        qp: QtGui.QPainter object
        
        '''
        ## initialize a pen with a red color, thickness of 1, solid line
        if self.prof.wdir.count() == 0:
            return
        pen = QtGui.QPen(self.trace_color, 1)
        pen.setStyle(QtCore.Qt.SolidLine)
  
        ## if there are missing values, get the mask
        try:
            mask = np.maximum(self.sru.mask, self.srv.mask)
            sru = self.sru[~mask]
            srv = self.srv[~mask]
            hgt = self.prof.hght[~mask]
        ## otherwise the data is fine
        except:
            sru = self.sru
            srv = self.srv
            hgt = self.prof.hght

        if len(sru) == 0 or len(srv) == 0 or len(hgt) == 0:
            return

        max_hght_idx = len(hgt) - 1
        while type(hgt[max_hght_idx]) == type(np.ma.masked):
            max_hght_idx -= 1

        interp_hght = np.arange(self.prof.hght[self.prof.sfc], min(hgt[max_hght_idx], self.hmax * 1000.), 10)
        interp_sru = np.interp(interp_hght, hgt[:(max_hght_idx + 1)], sru[:(max_hght_idx + 1)])
        interp_srv = np.interp(interp_hght, hgt[:(max_hght_idx + 1)], srv[:(max_hght_idx + 1)])
        sr_spd = np.hypot(interp_sru, interp_srv)

        qp.setPen(pen)
        for i in range(interp_hght.shape[0] - 1):
            spd1 = sr_spd[i]; spd2 = sr_spd[i+1]
            if utils.QC(spd1) and utils.QC(spd2):
                hgt1 = (interp_hght[i] - interp_hght[0]) / 1000; hgt2 = (interp_hght[i+1] - interp_hght[0]) / 1000
                ## convert the wind speeds to x pixels
                x1 = self.speed_to_pix(spd1); x2 = self.speed_to_pix(spd2)
                ## convert the height values to y pixels
                y1 = self.hgt_to_pix(hgt1); y2 = self.hgt_to_pix(hgt2)
                ## draw a line between the two points
                qp.drawLine(x1, y1, x2, y2)

        if utils.QC(self.srw_0_2km):       
            # Plot the 0-2 km mean SRW
            pen = QtGui.QPen(self.m0_2_color, 2)
            pen.setStyle(QtCore.Qt.SolidLine)
            qp.setPen(pen)
            x1 = self.speed_to_pix(self.srw_0_2km); x2 = self.speed_to_pix(self.srw_0_2km)
            y1 = self.hgt_to_pix(0.0); y2 = self.hgt_to_pix(2.0)
            qp.drawLine(x1, y1, x2, y2)

        if utils.QC(self.srw_4_6km):
            # Plot the 4-6 km mean SRW
            pen = QtGui.QPen(self.m4_6_color, 2)
            pen.setStyle(QtCore.Qt.SolidLine)
            qp.setPen(pen)
            x1 = self.speed_to_pix(self.srw_4_6km); x2 = self.speed_to_pix(self.srw_4_6km)
            y1 = self.hgt_to_pix(4.0); y2 = self.hgt_to_pix(6.0)
            qp.drawLine(x1, y1, x2, y2)
                    
        if utils.QC(self.srw_9_11km):
            # Plot the 9-11 km mean SRW
            pen = QtGui.QPen(self.m9_11_color, 2)
            pen.setStyle(QtCore.Qt.SolidLine)
            qp.setPen(pen)
            x1 = self.speed_to_pix(self.srw_9_11km); x2 = self.speed_to_pix(self.srw_9_11km)
            y1 = self.hgt_to_pix(9.0); y2 = self.hgt_to_pix(11.0)
            qp.drawLine(x1, y1, x2, y2)

if __name__ == '__main__':
    app_frame = QtGui.QApplication([])    
    tester = plotWinds()
    tester.show()    
    app_frame.exec_()



from qtpy.QtCore import *
from qtpy.QtGui import *

try:
    import Queue
except ImportError:
    import queue as Queue
import logging
import hashlib
from datetime import datetime
import traceback

class AsyncThreads(QObject):
    """
    AsyncThreads: A class to generate threads for whenever something can't run in the main process.
    """
    def __init__(self, max_threads, debug=False):
        super(AsyncThreads, self).__init__()
        self.threads = {}
        self.callbacks = {}
        self.debug = debug

        self.max_threads = max_threads
        self.running = 0
        self.queue = Queue.PriorityQueue(0)
        return

    def post(self, func, callback, *args, **kwargs):
        """
        Post a thread to be run.
        func:   The function to run in a separate thread
        callback:   The function to run once func() is done. It will be passed the output from func() as a single tuple.
        background [optional]: Boolean specifying whether to run in the background. Background processes are started only
            if there are no higher-priority processes to run. The default value is False.
        *args, **kwargs: Arguments to func()
        """
        thd_id = self._genThreadId()

        background = kwargs.get('background', False)
        if 'background' in kwargs:
            del kwargs['background']

        thd = self._threadFactory(func, thd_id, *args, **kwargs)
        thd.finished.connect(self.finish)

        priority = 1 if background else 0
        self.queue.put((priority, thd_id))

        self.threads[thd_id] = thd
        if callback is None:
            callback = lambda x: x
        self.callbacks[thd_id] = callback

        if not background or self.running < self.max_threads:
            self.startNext()
        return thd_id

    def isFinished(self, thread_id):
        """
        Returns a boolean specifying whether the thread corresponding to thread_id is finished.
        """
        return not (thread_id in self.threads)

    def join(self, thread_id):
        """
        Waits until the thread corresponding to thread_id is finished.
        """
        while not self.isFinished(thread_id):
            QCoreApplication.processEvents()

    def clearQueue(self):
        """
        Clears the queue of processes waiting to be started and kills any threads currently running.
        """
        self.queue = Queue.PriorityQueue(0)
        for thd_id, thd in self.threads.items():
            if thd.isRunning():
                thd.terminate()

        self.threads = {}
        self.callbacks = {}

    def startNext(self):
        try:
            prio, thd_id = self.queue.get(block=False)
        except Queue.Empty:
            return

        self.threads[thd_id].start()
        self.running += 1

    @Slot(str, tuple)
    def finish(self, thread_id, ret_val):
        thd = self.threads[thread_id]
        callback = self.callbacks[thread_id]

        callback(ret_val)

        del self.threads[thread_id]
        del self.callbacks[thread_id]

        self.running -= 1
        if self.running < self.max_threads:
            # Might not be the case if we just started a high-priority thread
            self.startNext()

    def _genThreadId(self):
        time_stamp = datetime.utcnow().isoformat()
        return hashlib.md5(time_stamp.encode('utf-8')).hexdigest()

    def _threadFactory(self, func, thread_id, *args, **kwargs):

        class AsyncThread(QThread):
            finished = Signal(str, tuple)

            def __init__(self, debug=False):
                super(AsyncThread, self).__init__()
                self.debug = debug            

            def run(self):
                try:
                    ret_val = func(*args, **kwargs)
                except Exception as e:
                    logging.exception(e)
                    if self.debug:
                        print(traceback.format_exc())
                    ret_val = e
                if type(ret_val) != tuple:
                    ret_val = (ret_val, )

                self.finished.emit(thread_id, ret_val)

        return AsyncThread(debug=self.debug)




try:
    from urllib2 import urlopen
except ImportError:
    from urllib.request import urlopen

import certifi
import re
import numpy as np
from datetime import datetime, timedelta

cache_len = timedelta(minutes=5)


goes_base_url = "http://sharp.weather.ou.edu/soundings/goes/"
goes_text = ""
goes_time = None

# SHARP OBSERVED AVAILBILITY
def _download_goes():
    global goes_time, goes_text
    now = datetime.utcnow()
    if goes_time is None or goes_time < now - cache_len:
        url_obj = urlopen(goes_base_url, cafile=certifi.where())
        goes_text = url_obj.read().decode('utf-8')
        goes_time = now

    return goes_text

def _available_goes(dt=None):
    '''
        _available_goes()

        Gets all of the available sounding times from the SHARP observed site.

        Returns
        -------
        matches : array
            Array of datetime objects that represents all the available times
            of sounding data on the SHARP site.
    '''
    text = _download_goes()

    matches = sorted(list(set(re.findall("([\d]{10})/", text))))
    return [ datetime.strptime(m, '%Y%m%d%H') for m in matches ]

def _availableat_goes(dt):
    '''
        _availableat_goes(dt)

        Get all the station locations where data was available for a certain dt object.

        Parameters
        ----------
        dt : datetime object

        Returns
        -------
        matches : array of strings
            An array that contains all of the three letter station identfiers.
    '''
    recent_url = "%s%s/available.txt" % (goes_base_url, dt.strftime('%Y%m%d%H'))
    text = urlopen(recent_url, cafile=certifi.where()).read().decode('utf-8')
    matches = re.findall("(.+).txt", text)
    return matches


sharp_base_url = "http://sharp.weather.ou.edu/soundings/obs/"
sharp_text = ""
sharp_time = None
sharp_archive_text = ""
sharp_archive_time = None

# SHARP OBSERVED AVAILBILITY
def _download_sharp():
    global sharp_time, sharp_text
    now = datetime.utcnow()
    if sharp_time is None or sharp_time < now - cache_len:
        url_obj = urlopen(sharp_base_url, cafile=certifi.where())
        sharp_text = url_obj.read().decode('utf-8')
        sharp_time = now
    return sharp_text

def _download_sharp_archive(dt):
    global sharp_time, sharp_text
    now = datetime.utcnow()
    base_url = 'http://sharp.weather.ou.edu/soundings/archive/%Y/%m/%d/'
    try:
        dt = datetime(dt.year(), dt.month(), dt.day(), 0,0,0)
    except:
        dt = dt
    if sharp_archive_time is None or sharp_archive_time < now - cache_len:
        url_obj = urlopen(dt.strftime(base_url), cafile=certifi.where())
        sharp_archive_text = url_obj.read().decode('utf-8')
        sharp_time = now
    return sharp_archive_text, dt

def _available_sharp(dt=None):
    '''
        _available_sharp()

        Gets all of the available sounding times from the SHARP observed site.

        Returns
        -------
        matches : array
            Array of datetime objects that represents all the available times
            of sounding data on the SHARP site.
    '''
    if dt is None:
        text = _download_sharp()
        matches = sorted(list(set(re.findall("([\d]{10})/", text))))
        return [ datetime.strptime(m, '%Y%m%d%H') for m in matches ]
    else:
        text, dt = _download_sharp_archive(dt)
        matches = sorted(list(set(re.findall(">([\d]{2})/<", text))))
        return [ datetime.strptime(dt.strftime('%Y%m%d') + m, '%Y%m%d%H')  for m in matches ]

def _availableat_sharp(dt):
    '''
        _availableat_sharp(dt)

        Get all the station locations where data was available for a certain dt object.

        Parameters
        ----------
        dt : datetime object

        Returns
        -------
        matches : array of strings
            An array that contains all of the three letter station identfiers.
    '''
    #recent_url = "%s%s/" % (sharp_base_url, dt.strftime('%Y%m%d%H/'))
    #text = urlopen(recent_url).read().decode('utf-8')
    #matches = re.findall("a href=\"(.+).txt\"", text)
    recent_url = 'http://sharp.weather.ou.edu/soundings/archive/%Y/%m/%d/%H/'
    text = urlopen(dt.strftime(recent_url), cafile=certifi.where()).read().decode('utf-8')
    matches = re.findall("a href=\"(.+).txt\"", text)
    return matches

##################################
##################################

# NUCAPS DATA AVAILABILITY - JTS
nucaps_conus_j01_url = "https://geo.nsstc.nasa.gov/SPoRT/jpss-pg/nucaps/gridded/conus/sharppy/j01/txt/"
nucaps_conus_aq0_url = "https://geo.nsstc.nasa.gov/SPoRT/jpss-pg/nucaps/gridded/conus/sharppy/aq0/txt/"
nucaps_conus_m01_url = "https://geo.nsstc.nasa.gov/SPoRT/jpss-pg/nucaps/gridded/conus/sharppy/m01/txt/"
nucaps_conus_m02_url = "https://geo.nsstc.nasa.gov/SPoRT/jpss-pg/nucaps/gridded/conus/sharppy/m02/txt/"
nucaps_conus_m03_url = "https://geo.nsstc.nasa.gov/SPoRT/jpss-pg/nucaps/gridded/conus/sharppy/m03/txt/"
nucaps_caribbean_j01_url = "https://geo.nsstc.nasa.gov/SPoRT/jpss-pg/nucaps/gridded/caribbean/sharppy/j01/txt/"
nucaps_alaska_j01_url = "https://geo.nsstc.nasa.gov/SPoRT/jpss-pg/nucaps/gridded/alaska/sharppy/j01/txt/"
nucaps_text = ""

##################################
# Retrieve the obs times for CONUS NOAA-20.
def _download_nucaps_conus_j01():
    global nucaps_text
    nucaps_text = urlopen(nucaps_conus_j01_url, cafile=certifi.where()).read().decode('utf-8')
    return nucaps_text

def _available_nucaps_conus_j01(dt=None):
    '''
        _available_nucaps_conus_j01()

        Gets all of the available sounding times from the SPoRT FTP site.

        Returns
        -------
        matches : array
            Array of datetime objects that represents all the available times
            of sounding data on the SPoRT FTP site.
    '''
    text = _download_nucaps_conus_j01()
    matches = sorted(list(set(re.findall(">([\d]{12})</a", text))))
    return [ datetime.strptime(m, '%Y%m%d%H%M') for m in matches ]


##################################
# Retrieve the obs times for CONUS Aqua.
def _download_nucaps_conus_aq0():
    global nucaps_text
    nucaps_text = urlopen(nucaps_conus_aq0_url, cafile=certifi.where()).read().decode('utf-8')
    return nucaps_text

def _available_nucaps_conus_aq0(dt=None):
    '''
        _available_nucaps_conus_aq0()

        Gets all of the available sounding times from the SPoRT FTP site.

        Returns
        -------
        matches : array
            Array of datetime objects that represents all the available times
            of sounding data on the SPoRT FTP site.
    '''
    text = _download_nucaps_conus_aq0()
    matches = sorted(list(set(re.findall(">([\d]{12})</a", text))))
    return [ datetime.strptime(m, '%Y%m%d%H%M') for m in matches ]


#################################
# Retrieve the obs times for CONUS MetOp-B.
def _download_nucaps_conus_m01():
    global nucaps_text
    nucaps_text = urlopen(nucaps_conus_m01_url, cafile=certifi.where()).read().decode('utf-8')
    return nucaps_text

def _available_nucaps_conus_m01(dt=None):
    '''
        _available_nucaps_conus_m01()

        Gets all of the available sounding times from the SPoRT FTP site.

        Returns
        -------
        matches : array
            Array of datetime objects that represents all the available times
            of sounding data on the SPoRT FTP site.
    '''
    text = _download_nucaps_conus_m01()
    matches = sorted(list(set(re.findall(">([\d]{12})</a", text))))
    return [ datetime.strptime(m, '%Y%m%d%H%M') for m in matches ]


#################################
# Retrieve the obs times for CONUS MetOp-A.
def _download_nucaps_conus_m02():
    global nucaps_text
    nucaps_text = urlopen(nucaps_conus_m02_url, cafile=certifi.where()).read().decode('utf-8')
    return nucaps_text

def _available_nucaps_conus_m02(dt=None):
    '''
        _available_nucaps_conus_m02()

        Gets all of the available sounding times from the SPoRT FTP site.

        Returns
        -------
        matches : array
            Array of datetime objects that represents all the available times
            of sounding data on the SPoRT FTP site.
    '''
    text = _download_nucaps_conus_m02()
    matches = sorted(list(set(re.findall(">([\d]{12})</a", text))))
    return [ datetime.strptime(m, '%Y%m%d%H%M') for m in matches ]


#################################
# Retrieve the obs times for CONUS MetOp-C.
def _download_nucaps_conus_m03():
    global nucaps_text
    nucaps_text = urlopen(nucaps_conus_m03_url, cafile=certifi.where()).read().decode('utf-8')
    return nucaps_text

def _available_nucaps_conus_m03(dt=None):
    '''
        _available_nucaps_conus_m03()

        Gets all of the available sounding times from the SPoRT FTP site.

        Returns
        -------
        matches : array
            Array of datetime objects that represents all the available times
            of sounding data on the SPoRT FTP site.
    '''
    text = _download_nucaps_conus_m03()
    matches = sorted(list(set(re.findall(">([\d]{12})</a", text))))
    return [ datetime.strptime(m, '%Y%m%d%H%M') for m in matches ]


#################################
# Retrieve the obs times for Caribbean NOAA-20.
def _download_nucaps_caribbean_j01():
    global nucaps_text
    nucaps_text = urlopen(nucaps_caribbean_j01_url, cafile=certifi.where()).read().decode('utf-8')
    return nucaps_text

def _available_nucaps_caribbean_j01(dt=None):
    '''
        _available_nucaps_caribbean_j01()

        Gets all of the available sounding times from the SPoRT FTP site.

        Returns
        -------
        matches : array
            Array of datetime objects that represents all the available times
            of sounding data on the SPoRT FTP site.
    '''
    text = _download_nucaps_caribbean_j01()
    matches = sorted(list(set(re.findall(">([\d]{12})</a", text))))
    return [ datetime.strptime(m, '%Y%m%d%H%M') for m in matches ]


#################################
# Retrieve the obs times for Alaska NOAA-20.
def _download_nucaps_alaska_j01():
    global nucaps_text
    nucaps_text = urlopen(nucaps_alaska_j01_url, cafile=certifi.where()).read().decode('utf-8')
    return nucaps_text

def _available_nucaps_alaska_j01(dt=None):
    '''
        _available_nucaps_alaska_j01()

        Gets all of the available sounding times from the SPoRT FTP site.

        Returns
        -------
        matches : array
            Array of datetime objects that represents all the available times
            of sounding data on the SPoRT FTP site.
    '''
    text = _download_nucaps_alaska_j01()
    matches = sorted(list(set(re.findall(">([\d]{12})</a", text))))
    return [ datetime.strptime(m, '%Y%m%d%H%M') for m in matches ]


##################################
##################################

# SPC DATA AVAILABLILY
spc_base_url = "http://www.spc.noaa.gov/exper/soundings/"
spc_text = ""
spc_time = None

def _download_spc():
    global spc_time, spc_text
    now = datetime.utcnow()
    if spc_time is None or spc_time < now - cache_len:
        url_obj = urlopen(spc_base_url, cafile=certifi.where())
        spc_text = url_obj.read().decode('utf-8')

        spc_time = now

    return spc_text

def _available_spc(dt=None):
    '''
        _available_spc()

        Gets all of the available sounding times from the SPC site.

        Returns
        -------
        matches : array
            Array of datetime objects that represents all the available times
            of sounding data on the SPC site.
    '''
    text = _download_spc()
    matches = sorted(list(set(re.findall("([\d]{8})_OBS", text))))
    return [ datetime.strptime(m, '%y%m%d%H') for m in matches ]

def _availableat_spc(dt):
    '''
        _availableat_spc(dt)

        Get all the station locations where data was available for a certain dt object.

        Parameters
        ----------
        dt : datetime object

        Returns
        -------
        matches : array of strings
            An array that contains all of the three letter station identfiers.
    '''
    recent_url = "%s%s/" % (spc_base_url, dt.strftime('%y%m%d%H_OBS'))
    text = urlopen(recent_url, cafile=certifi.where()).read().decode('utf-8')
    matches = re.findall("show_soundings\(\"([\w]{3}|[\d]{5})\"\)", text)
    return matches

### IEM BUFKIT ARCHIVE CODE
iem_base_url = "http://mtarchive.geol.iastate.edu/%Y/%m/%d/bufkit/%H/"
iem_text = ""
iem_time = None

### PSU CODE
psu_base_url = "ftp://ftp.meteo.psu.edu/pub/bufkit/"
psu_text = ""
psu_time = None

def _download_psu():
    '''
        _download_psu()

        Downloads the PSU directory webpage that lists all the
        files available.

        Returns
        -------
        psu_text : string
            Lists the files within the PSU FTP site.
    '''
    global psu_time, psu_text
    now = datetime.utcnow()
    if psu_time is None or psu_time < now - cache_len:
        url_obj = urlopen(psu_base_url, cafile=certifi.where())
        psu_text = url_obj.read().decode('utf-8')

        psu_time = now

    return psu_text

def _availableat_psu(model, dt):
    '''
        _availableat_psu

        Downloads a list of all the BUFKIT profile stations for a given
        model and runtime.

        Parameters
        ----------
        model : string
            A string representing the forecast model requested
        dt : datetime object
            A datetime object that represents the model initalization time.
    '''
    if model == '4km nam': model = 'nam4km'
    _repl = {'gfs':'gfs3', 'nam':'namm?', 'rap':'rap', 'nam4km':'nam4kmm?', 'hrrr':'hrrr', 'sref':'sref'}

    cycle = dt.hour
    url = "%s%s/%02d/" % (psu_base_url, model.upper(), cycle)
    url_obj = urlopen(url, cafile=certifi.where())
    text = url_obj.read().decode('utf-8')

    stns = re.findall("%s_(.+)\.buf" % _repl[model], text)
    return stns

def _available_psu(model, dt=None):
    '''
        _available_psu

        Downloads a list of datetime objects that represents
        the available model times from the PSU server.

        Parameters
        ----------
        model : string
            the name of the forecast model
        nam : boolean (default: false)
            Specifies whether or not this is the NAM or 4km NAM
        off : boolean (default: false)
            Specifies whether or not this is an off-hour run
    '''

    if dt is not None and dt < datetime.utcnow() - timedelta(3600*29): # We know PSU is only a 24 hr service
        return []

    if model == '4km nam': model = 'nam4km'

    psu_text = _download_psu()
    latest = re.search("%s\.([\d]{12})\.done" % model, psu_text).groups(0)[0]
    dt = datetime.strptime(latest, "%Y%m%d%H%M")
    return [ dt ]

### IEM CODE
iem_base_url = "http://mtarchive.geol.iastate.edu/%Y/%m/%d/bufkit/%H/MODEL/"
iem_text = ""
iem_time = None

def _download_iem():
    '''
        _download_iem()

        Downloads the IEM directory webpage that lists all the
        files available.

        Returns
        -------
        iem_text : string
            Lists the files within the IEM site.
    '''
    global iem_time, iem_text
    now = datetime.utcnow()
    if iem_time is None or iem_time < now - cache_len:
        iem_obj = urlopen(iem_base_url, cafile=certifi.where())
        psu_text = url_obj.read().decode('utf-8')
        iem_time = now

    return iem_text

def _availableat_iem(model, dt):
    '''
        _availableat_iem

        Downloads a list of all the BUFKIT profile stations for a given
        model and runtime.

        Parameters
        ----------
        model : string
            A string representing the forecast model requested
        dt : datetime object
            A datetime object that represents the model initalization time.
    '''
    if model == '4km nam' or model == 'nam nest': model = 'nam4km'
    _repl = {'gfs':'gfs3', 'nam':'namm?', 'rap':'rap', 'nam4km':'nam4kmm?', 'hrrr':'hrrr', 'sref':'sref', 'ruc':'ruc'}

    cycle = dt.hour
    url = dt.strftime(iem_base_url).replace("MODEL", model.lower())
    url_obj = urlopen(url, cafile=certifi.where())
    text = url_obj.read().decode('utf-8')

    stns = re.findall("%s_(.+)\.buf\">" % _repl[model], text)

    return stns

def _available_iem(model, dt=None):
    '''
        _available_iem

        Downloads a list of datetime objects that represents
        the available model times from the PSU server.

        Parameters
        ----------
        model : string
            the name of the forecast model
        nam : boolean (default: false)
            Specifies whether or not this is the NAM or 4km NAM
        off : boolean (default: false)
            Specifies whether or not this is an off-hour run
    '''
    #nam=(m in [ 'nam', '4km nam' ])
    if dt is None:
        dt = datetime.utcnow()
    else:
        try:
            a = int(dt.year)
        except:
            dt = datetime(dt.year(), dt.month(), dt.day())

    if model == '4km nam' or model == 'nam nest': model = 'nam4km'

    # Filtering out datetimes where we know there is no data on the IEM server.
    # Either due to no data, depreciated modeling systems, etc.
    if dt < datetime(2010,12,30): # No Bufkit files before this date
        return []
    if dt > datetime.utcnow() - timedelta(seconds=3600*24):
        return []
    if model == 'ruc' and dt > datetime(2012,5,1,11,0,0): # RIP RUC
        return []
    if model == 'nam4km' and dt < datetime(2013,3,25,0,0,0): # No NAM 4 km data before this time
        return []
    if model == 'rap' and dt < datetime(2012,5,1): # No RAP data prior to this date
        return []
    if model == 'hrrr' and dt < datetime(2019,8,24):
        return []

    if model == 'ruc' or model == 'rap':
        if dt.year == 2012 and dt.month == 5 and dt.day == 1: # Need to truncate the times since there was a switchover from RUC to RAP on this day.
            if model == 'ruc':
                start = 0; end = 12;
            elif model == 'rap':
                start = 12; end = 24;
        else:
            start = 0; end = 24;
        inc = 1

    if model == 'gfs' or model == 'nam' or model == 'nam4km':
        start = 0; end = 24; inc = 6

    if model == 'hrrr':
        if dt.year == 2019 and dt.month == 8 and dt.day == 24:
            start = 1
        else:
            start = 0
        end = 24
        inc = 1

    return [datetime(dt.year, dt.month, dt.day, h, 0, 0) for h in np.arange(start,end,inc)]


## PECAN MAPS ENSEMBLE CODE
pecan_base_url = 'http://weather.ou.edu/~map/real_time_data/PECAN/'
#http://weather.ou.edu/~map/real_time_data/PECAN/2015061112/soundings/TOP_2015061113.txt

def _available_oupecan(**kwargs):
    text = urlopen(pecan_base_url, cafile=certifi.where()).read().decode('utf-8')
    matches = sorted(list(set(re.findall("([\d]{10})", text))))
    return [ datetime.strptime(m, "%Y%m%d%H") for m in matches ]

def _availableat_oupecan(dt):
    dt_string = datetime.strftime(dt, '%Y%m%d%H')
    url = "%s%s/soundings/" % (pecan_base_url, dt_string)
    url_obj = urlopen(url, cafile=certifi.where())
    text = url_obj.read().decode('utf-8')
    dt_string = datetime.strftime(dt, '%Y%m%d%H')
    stns = re.findall("([\w]{3})_%s.txt" % dt_string, text)
    return np.unique(stns)

## NCAR ENSEMBLE CODE
ncarens_base_url = 'http://sharp.weather.ou.edu/soundings/ncarens/'

def _available_ncarens(dt=None):
    text = urlopen(ncarens_base_url, cafile=certifi.where()).read().decode('utf-8')

    matches = sorted(list(set(re.findall("([\d]{8}_[\d]{2})", text))))
    return [ datetime.strptime(m, '%Y%m%d_%H') for m in matches ]

def _availableat_ncarens(dt):
    dt_string = datetime.strftime(dt, '%Y%m%d_%H')
    url = "%s%s/" % (ncarens_base_url, dt_string)
    url_obj = urlopen(url, cafile=certifi.where())
    text = url_obj.read().decode('utf-8')

    stns = re.findall("(N[\w]{2}.[\w]{2}W.[\w]{2,3}.[\w]{2}).txt", text)
    stns2 = re.findall("([\w]{3}).txt", text)
    return stns + stns2

def _available_nssl(ens=False):
    path_to_nssl_wrf = ''
    path_to_nssl_wrf_ens = ''

# A dictionary of the available times for profiles from observations, forecast models, NUCAPS, etc.
# JTS - Added the available obs times for the various NUCAPS sources.
available = {
    'psu':{},
    'iem':{},
    'spc':{'observed': lambda dt=None: _available_spc(dt=dt)},
#    'ou_pecan': {'pecan ensemble': lambda dt=None: _available_oupecan(dt=dt) },
#    'ncar_ens': {'ncar ensemble': lambda dt=None: _available_ncarens(dt=dt) },
    'sharp': {'ncar ensemble': lambda dt=None: _available_ncarens(dt=dt), 'observed': lambda dt=None: _available_sharp(dt=dt), 'goes': lambda dt=None: _available_goes(dt=dt)},
    'local': {'local wrf-arw': lambda filename:  _available_local(filename)},
    'stc': {'nucaps conus noaa-20': lambda dt=None: _available_nucaps_conus_j01(dt=dt), \
            'nucaps conus aqua': lambda dt=None: _available_nucaps_conus_aq0(dt=dt), \
            'nucaps conus metop-b': lambda dt=None: _available_nucaps_conus_m01(dt=dt), \
            'nucaps conus metop-a': lambda dt=None: _available_nucaps_conus_m02(dt=dt), \
            'nucaps conus metop-c': lambda dt=None: _available_nucaps_conus_m03(dt=dt), \
            'nucaps caribbean noaa-20': lambda dt=None: _available_nucaps_caribbean_j01(dt=dt), \
            'nucaps alaska noaa-20': lambda dt=None: _available_nucaps_alaska_j01(dt=dt)},
}

# A dictionary containing paths to the stations for different observations, forecast models, etc.
# given a specific datetime object.
availableat = {
    'psu':{},
    'iem':{},
    'spc':{'observed':_availableat_spc},
    'ou_pecan': {'pecan ensemble': lambda dt: _availableat_oupecan(dt)},
    'sharp': {'ncar ensemble': lambda dt: _availableat_ncarens(dt) , 'observed':_availableat_sharp, 'goes':_availableat_goes},
}

# Set the available and available-at-time functions for the PSU data.
for model in [ 'gfs', 'nam', 'rap', 'hrrr', '4km nam', 'sref' ]:
    available['psu'][model] = (lambda m: lambda dt=None: _available_psu(m, dt=dt))(model)
    availableat['psu'][model] = (lambda m: lambda dt: _availableat_psu(m, dt))(model)

# Set the available and available-at-time functions for the IEM data.
for model in [ 'gfs', 'nam', 'rap', 'ruc', 'nam nest','hrrr' ]:
    available['iem'][model] = (lambda m: lambda dt=None: _available_iem(m, dt=dt))(model)
    availableat['iem'][model] = (lambda m: lambda dt: _availableat_iem(m, dt))(model)

if __name__ == "__main__":

    dt = datetime.utcnow()
    dt = available['spc']['observed'](dt)
    print(dt)
    print(availableat['spc']['observed'](dt[-1]))
    ##stop
    dt = datetime(2015,1,1,0,0,0)
    print(dt)
    dt = available['iem']['gfs'](dt)
    stns = availableat['iem']['gfs'](dt[0])
    print(dt, stns)
    #dt = available['spc']['observed']()
    #stns = availableat['spc']['observed'](dt[-1])
    dt = available['ou_pecan']['pecan ensemble']()
    #print(dt)
    #stns = availableat['ou_pecan']['pecan ensemble'](dt[-2])
    #print(stns)



#!/usr/bin/env python
import os.path
import subprocess
release = True
__version__ = '1.4.0'
__version_name__ = "Andover"

_repository_path = os.path.split(__file__)[0]
_git_file_path = os.path.join(_repository_path, '__git_version__.py')


def _minimal_ext_cmd(cmd):
    # construct minimal environment
    env = {}
    for k in ['SYSTEMROOT', 'PATH']:
        v = os.environ.get(k)
        if v is not None:
            env[k] = v
    # LANGUAGE is used on win32
    env['LANGUAGE'] = 'C'
    env['LANG'] = 'C'
    env['LC_ALL'] = 'C'
    out = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                           env=env).communicate()[0]
    return out

def get_git_branch():
    '''
    Gets the current Git branch.

    '''
    try:
        out = _minimal_ext_cmd(['git', 'rev-parse', '--abbrev-ref', 'HEAD'])
        branch = out.strip().decode('ascii')
    except:
        branch = ''
    return branch


def get_git_hash():
    '''
    Gets the last GIT commit hash and date for the repository, using the
    path to this file.

    '''
    try:
        out = _minimal_ext_cmd(['git', 'rev-parse', '--short', 'HEAD'])
        revision = out.strip().decode('ascii')
    except:
        revision = ''
    return revision


def get_git_date(git_hash):
    '''
    Gets the date of the last commit.

    '''
    try:
        out = _minimal_ext_cmd(['git', 'show', git_hash, '--date=short',
                                '--format="%ad"'])
        date = out.strip().decode('ascii').split('"')[1]
    except:
        date = ''
    return date


def get_git_revision():
    git_branch = get_git_branch()
    git_hash = get_git_hash()
    git_date = get_git_date(git_hash)

    if git_branch:
        rev = '.dev0+%s-%s(%s)' % (git_branch, git_hash, git_date)
    else:
        rev = ''
    return rev


def write_git_version():
    '''
    Write the GIT revision to a file.

    '''
    rev = get_git_revision()
    if rev == "":
        if os.path.isfile(_git_file_path):
            return
    gitfile = open(_git_file_path, 'w')
    gitfile.write('rev = "%s"\n' % rev)
    gitfile.close()


def get_version():
    '''
    Get the version of the package, including the GIT revision if this
    is an actual release.

    '''
    version = __version__
    if not release:
        try:
            from . import __git_version__
            version += __git_version__.rev
        except ImportError:
            version += get_git_revision()
    return version





if __name__ == "__main__":
    write_git_version()



import numpy as np
from qtpy import QtGui, QtCore, QtWidgets
import sharppy.sharptab as tab
from sharppy.sharptab.constants import *

## routine written by Kelton Halbert
## keltonhalbert@ou.edu

__all__ = ['backgroundSpeed', 'plotSpeed']

class backgroundSpeed(QtWidgets.QFrame):
    '''
    Handles drawing the plot background.
    '''
    def __init__(self):
        super(backgroundSpeed, self).__init__()
        self.initUI()


    def initUI(self):
        ## initialize frame variables, such as length, width, and
        ## expected max/min values of data
        self.lpad = 0; self.rpad = 0
        self.tpad = 0; self.bpad = 20
        self.wid = self.size().width() - self.rpad
        self.hgt = self.size().height() - self.bpad
        self.tlx = self.rpad; self.tly = self.tpad
        self.brx = self.wid; self.bry = self.hgt
        ## set the max and min pressure expected, and convert it to log space
        self.pmax = 1050.; self.pmin = 100.
        self.log_pmax = np.log(self.pmax); self.log_pmin = np.log(self.pmin)
        ## set the max/min wind speed expected
        if self.wind_units == "knots":
            self.smax = 140.; self.smin = 0. # knots
            self.delta = 20.
        elif self.wind_units == 'm/s':
            self.smax = 80.; self.smin = 0. # m/s
            self.delta = 10.
        
        self.font_ratio = 0.12#0512
        fsize = round(self.size().width() * self.font_ratio) + 2
        self.label_font = QtGui.QFont('Helvetica', fsize)
        self.plotBitMap = QtGui.QPixmap(self.width(), self.height())
        self.plotBitMap.fill(QtGui.QColor(self.bg_color))
        self.plotBackground()

    def resizeEvent(self, e):
        '''
        Handles when the window is resized.
        '''
        self.initUI()
    
    def plotBackground(self):
        '''
        Handles painting the frame.
        '''
        ## initialize a QPainter object
        qp = QtGui.QPainter()
        qp.begin(self.plotBitMap)
        qp.setRenderHint(qp.Antialiasing)
        qp.setRenderHint(qp.TextAntialiasing)
        ## draw the background frame
        self.draw_frame(qp)
        ## draw the vertical ticks for wind speed
        for s in range(int(self.smin),int(self.smax),int(self.delta)):
            if s % (int(self.delta)*2) == 0 and s != 0:
                label=True
            else:
                label=False
            self.draw_speed(s, qp, int(self.delta), label)
        ## Draw the title and units
        pen = QtGui.QPen(QtGui.QColor(self.fg_color), 1, QtCore.Qt.DashLine)
        qp.setPen(pen)

        fsize = round(self.size().width() * self.font_ratio)
        self.title_font = QtGui.QFont('Helvetica', fsize+1)
        qp.setFont(self.title_font)
        qp.drawText(self.tlx+2, self.tly+2, self.brx-self.tlx, 30,
                   QtCore.Qt.AlignTop | QtCore.Qt.AlignLeft | QtCore.Qt.TextWordWrap, "Wind Speed\n(" + self.wind_units + ")")
        qp.end() 


    def draw_frame(self, qp):
        '''
        Draws the frame boarders.
        '''
        ## initialize a pen with white color, thickness 2, solid line
        pen = QtGui.QPen(QtGui.QColor(self.fg_color), 2, QtCore.Qt.SolidLine)
        qp.setPen(pen)
        qp.drawLine(self.tlx, self.tly, self.brx, self.tly)
        qp.drawLine(self.brx, self.tly, self.brx, self.bry)
        qp.drawLine(self.brx, self.bry, self.tlx, self.bry)
        qp.drawLine(self.tlx, self.bry, self.tlx, self.tly)


    def draw_speed(self, s, qp, delta=0, drawlabel=True):
        '''
        Draw background speed ticks.
        --------
        s: wind speed
        qp: QtGui.QPainter object
        
        '''
        ## initialize a pen with an orange/brown color, thickness 1, dashed line
        pen = QtGui.QPen(self.isotach_color, 1, QtCore.Qt.DashLine)
        qp.setPen(pen)
        qp.setFont(self.label_font)
        ## convert the speed value to pixel coordinates
        x1 = self.speed_to_pix(s)
        labelx1 = self.speed_to_pix(s - delta) # e.g. 20 - 20 = 0, 0 to 40
        label_width = self.speed_to_pix(s+delta) - self.speed_to_pix(s-delta)
        ## draw a dashed line of constant wind speed value
        qp.drawLine(x1, self.bry, x1, self.tly)
        if drawlabel is True and s > 0:
            pen = QtGui.QPen(QtGui.QColor(self.fg_color), 1, QtCore.Qt.DashLine)
            qp.setPen(pen)
            qp.drawText(labelx1, self.bry+5, label_width, 10,
                   QtCore.Qt.AlignTop | QtCore.Qt.AlignCenter, str(int(s)))
    
    def pres_to_pix(self, p):
        '''
        Function to convert a pressure value to a Y pixel.
        
        '''
        scl1 = self.log_pmax - self.log_pmin
        scl2 = self.log_pmax - np.log(p)
        return self.bry - (scl2 / scl1) * (self.bry - self.tpad)

    def speed_to_pix(self, s):
        '''
        Function to convert a wind speed value to a X pixel.
        '''
        scl1 = self.smax - self.smin
        scl2 = self.smax - s
        return self.brx - (scl2 / scl1) * (self.brx - self.rpad)

class plotSpeed(backgroundSpeed):
    '''
    Handles plotting the data in the frame.
    '''
    def __init__(self):
        self.bg_color = '#000000'
        self.fg_color = '#ffffff'
        self.wind_units = 'knots'
        self.isotach_color = QtGui.QColor("#9D5736")

        super(plotSpeed, self).__init__()
        ## initialize values to be accessable to functions
        self.prof = None

        ## give different colors for different height values.
        ## these are consistent with the hodograph colors.
        self.low_level_color = QtGui.QColor("#FF0000")
        self.mid_level_color = QtGui.QColor("#00FF00")
        self.upper_level_color = QtGui.QColor("#FFFF00")
        self.trop_level_color = QtGui.QColor("#00FFFF")

    def setProf(self, prof):
        self.prof = prof
        self.u = prof.u; self.v = prof.v
        self.hght = prof.hght; self.pres = prof.pres

        self.clearData()
        self.plotBackground()
        self.plotData()
        self.update()

    def setPreferences(self, update_gui=True, **prefs):
        self.bg_color = prefs['bg_color']
        self.fg_color = prefs['fg_color']
        self.wind_units = prefs['wind_units']
        
        if self.wind_units == "knots":
            self.smax = 140.; self.smin = 0. # knots
            self.delta = 20.
        elif self.wind_units == 'm/s':
            self.smax = 80.; self.smin = 0. # m/s
            self.delta = 10.
           
        self.low_level_color = QtGui.QColor(prefs['0_3_color'])
        self.mid_level_color = QtGui.QColor(prefs['3_6_color'])
        self.upper_level_color = QtGui.QColor(prefs['6_9_color'])
        self.trop_level_color = QtGui.QColor(prefs['9_12_color'])

        self.isotach_color = QtGui.QColor(prefs['spd_itach_color'])

        if update_gui:
            self.clearData()
            self.plotBackground()
            self.plotData()
            self.update()

    def resizeEvent(self, e):
        '''
        Handles when the window is resized.
        '''
        super(plotSpeed, self).resizeEvent(e)
        self.plotData()
    
    def paintEvent(self, e):
        super(plotSpeed, self).paintEvent(e)
        qp = QtGui.QPainter()
        qp.begin(self)
        qp.drawPixmap(0, 0, self.plotBitMap)
        qp.end()

    def clearData(self):
        '''
        Handles the clearing of the pixmap
        in the frame.
        '''
        self.plotBitMap = QtGui.QPixmap(self.width(), self.height())
        self.plotBitMap.fill(QtGui.QColor(self.bg_color))
    
    def plotData(self):
        '''
        Handles drawing the data on the frame.
        '''
        ## initialize a QPainter object

        if self.prof is None:
            return

        qp = QtGui.QPainter()
        qp.begin(self.plotBitMap)
        qp.setRenderHint(qp.Antialiasing)
        qp.setRenderHint(qp.TextAntialiasing)
        ## draw the wind speed profile
        self.draw_profile(qp)
        qp.end()

    def draw_profile(self, qp):
        '''
        Draw the Speed vs. Height profile.
        --------
        qp: QtGui.QPainter object
        '''
        ## initialize a pen starting with the low level color,
        ## thickness of 2, solid line.
        pen = QtGui.QPen(self.low_level_color, 1)
        pen.setStyle(QtCore.Qt.SolidLine)
        ## if there are missing values, get the data mask
        try:
            mask = np.maximum(np.maximum(self.u.mask, self.v.mask), self.hght.mask)
            hgt = tab.interp.to_agl(self.prof, self.hght[~mask])
            pres = self.pres[~mask]
            u = self.u[~mask]
            v = self.v[~mask]
            ## calculate teh wind speed
            spd = np.sqrt( u**2 + v**2 )
        ## otherwise, the data is fine.
        except:
            hgt = tab.interp.to_agl(self.prof, self.hght)
            pres = self.pres
            u = self.u; v = self.v
            ## calculate the windspeed
            spd = np.sqrt( u**2 + v**2 )

        if self.wind_units == 'm/s':
            spd = tab.utils.KTS2MS(spd)

        ## loop through the profile
        for i in range( pres.shape[0] ):
            ## get the important values from the profile
            hgt1 = hgt[i]
            p1 = pres[i]
            spd1 = spd[i]
            ## convert the speed to x pixel coordinates
            ## and convert the pressure in log space to a
            ## y pixel coordinate
            x1 = self.speed_to_pix(spd1)
            y1 = self.pres_to_pix(p1)
            ## now color code the different heights
            if hgt1 < 3000:
                pen = QtGui.QPen(self.low_level_color, 2)
            elif hgt1 < 6000:
                pen = QtGui.QPen(self.mid_level_color, 2)
            elif hgt1 < 9000:
                pen = QtGui.QPen(self.upper_level_color, 2)
            else:
                pen = QtGui.QPen(self.trop_level_color, 2)
            ## Draw a horizontal line with the length of the wind speed
            qp.setPen(pen)
            qp.drawLine(0, y1, x1, y1)

if __name__ == '__main__':
    app_frame = QtGui.QApplication([])    
    tester = plotSpeed()
    tester.show()    
    app_frame.exec_()

